/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2007,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

#include <spu_mfcio.h>

#include <libisolation.h>
#include <secure_env.h>

vector unsigned int orig_msg = {0x60000000, 0x00000000, 0x00000000, 0x00000000};

int main(unsigned long long id __attribute__ ((unused)), unsigned long long argv __attribute__ ((unused)), unsigned long long env __attribute__ ((unused)), void *secure_env)
{
  secure_env_t *env_ptr = (secure_env_t*)secure_env;  
  vector unsigned int tmp = orig_msg;
  vector unsigned int decrypted = (vector unsigned int)spu_splats(0);
  uint64_t shared_buf_addr;
  uint32_t shared_buf_addr_high;
  uint32_t shared_buf_addr_low;
  
  /* Read address for shared buffer */
  shared_buf_addr_high = spu_read_in_mbox();
  shared_buf_addr_low = spu_read_in_mbox();
  shared_buf_addr = mfc_hl2ea(shared_buf_addr_high,shared_buf_addr_low);
  
  /* Encrypt the orignal message, and copy the encrypted message to the main memory */
  if (encrypt_out(shared_buf_addr, &tmp, sizeof(tmp), env_ptr->app_version_specific_key) != 0) {
    return -1;
  } 
    
  /* Copy the encrypted message into the isolated LS, and decrypt it */
  if (decrypt_in(&decrypted, shared_buf_addr, sizeof(decrypted), env_ptr->app_version_specific_key) != 0) {
    return -2;
  }
      
  /* Compare the orignal message with the decrypted message */
   vector unsigned int t = spu_cmpeq(orig_msg, decrypted);
   if (spu_extract(spu_gather(spu_nor(t,t)), 0)) {
         return -3;
   }
    
  return 0;
}
