/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2007,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <libspe2.h>
#include <pthread.h>

extern spe_program_handle_t encrypted_spu;

/* arguments list of a PPU thread */
typedef struct ppu_pthread_data {
    spe_context_ptr_t spe_id;
    pthread_t pthread;
    unsigned int entry;
    unsigned int flags;
    void *argp;
    void *envp;
    spe_stop_info_t stopinfo;
} ppu_pthread_data_t;


/**
 * The entry point of a PPU thread to start an SPE application. 
 */
void *ppu_pthread_function(void *arg) {
    ppu_pthread_data_t *datap = (ppu_pthread_data_t *)arg;
    int rc;
    do {
        if((rc = spe_context_run(datap->spe_id, &datap->entry, datap->flags, datap->argp, datap->envp, &datap->stopinfo)) < 0) {
            perror("failed running context");
            exit(1);
        }
    } while( rc > 0 ); // loop until exit
    pthread_exit(NULL);
}

/**
 * Entry point of 'iso_encrypted' 
 */
int main(int argc __attribute__ ((unused)), char **argv __attribute__ ((unused)) )
{
    ppu_pthread_data_t data;
    int rc;

    /* Create an SPE-threads in isolation mode to execute 'encrypted_spu'. */
    /* Create conext */
    if ((data.spe_id = spe_context_create (SPE_ISOLATE_EMULATE, NULL)) == NULL) {
        fprintf(stderr, "Failed spe_context_create(errno=%d)\n", errno);
        /* a bug in an older kernel may cause libspe to return EFAULT instead of ENODEV */
        if(errno == ENODEV) {
            perror("Failed creating context, requires an SPE enabled for isolation mode");
        } else {
            perror("Failed creating context");
        }    
        exit(1); 
    }
    
    /* Load program */
    if ((rc = spe_program_load(data.spe_id, &encrypted_spu)) != 0) {
        fprintf(stderr, "Failed spe_program_load(errno=%d)\n", errno);
        perror("Failed loading program");
        exit(1); 
    }
    
    /* Initialize data */
    data.entry = SPE_DEFAULT_ENTRY;
    data.flags = 0;
    data.argp = NULL;
    data.envp = NULL;
    
    /* Create thread */
    if ((rc = pthread_create(&data.pthread, NULL, &ppu_pthread_function, &data)) != 0) {
        fprintf(stderr, "Failed pthread_create(errno=%d)\n", errno);
        perror("Failed creating thread");
        exit(1);
    }

    /* Wait for SPU-thread to complete execution. */
    /* Join thread */
    if ((rc = pthread_join (data.pthread, NULL)) != 0) {
        fprintf(stderr, "Failed pthread_join(rc=%d, errno=%d\n", rc, errno);
        perror("failed joining thread");
        exit(1);
    }
    
    /* Destroy context */
    if ((rc = spe_context_destroy (data.spe_id)) != 0) {
        fprintf(stderr, "Failed spe_context_destroy(rc=%d, errno=%d)\n", rc, errno);
        perror("failed destroying thread");
        exit(1);
    }

    /* Check stop reason of the SPE application */
    if (data.stopinfo.stop_reason == SPE_EXIT) {
        printf("spe thread has exited(stopinfo.spe_exit_code=%d)\n", data.stopinfo.result.spe_exit_code);
        return (data.stopinfo.result.spe_exit_code);
    } else {
        fprintf(stderr, "stopinfo.stop_reason=%x, stopinfo.spe_exit_code=%x \n", data.stopinfo.stop_reason, data.stopinfo.result.spe_exit_code);
        return -1;
    }
}
