/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2007,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

#include <spu_intrinsics.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <libisolation.h>

#include "test_bin.h"

int posix_scenario()
{
    struct stat stat_buf;
	unsigned char data_buf[TEST_FILE_LEN];
	int filedes, count;
	char *test_filename_tmp = "posix_testfile.tmp";
	
    // create test file
    filedes = open(test_filename_tmp, O_WRONLY | O_CREAT);
    if (filedes == -1) {
		perror("error opening tmp file for writing");
		return -1;
    }
    
    // write out test data
    count = write(filedes, reference_file_contents, TEST_FILE_LEN);
    if (count != TEST_FILE_LEN) {
		fprintf(stderr, "error - expected to write %d bytes to %s, but wrote %d, errno=%d, error=\"", TEST_FILE_LEN, test_filename_tmp, count, errno);
		fputs(strerror(errno), stderr);
		fputs("\"\n", stderr);
		return -1;
    }
    
    // close test file
    if (0 != close(filedes)) {
		perror("error closing tmp file");
		return -1;
    }
    
    // check size of created test file before opening it
    if (0 != stat(test_filename_tmp, &stat_buf)) {
		perror("calling stat() failed\n");
		return -1;
    }
    if (stat_buf.st_size != TEST_FILE_LEN) {
		fprintf(stderr, "error - length of file %s is %d, must be %d\n", test_filename_tmp, (int) stat_buf.st_size, TEST_FILE_LEN);
		return -1;
    }

    // open created test file for reading
    filedes = open(test_filename_tmp, O_RDONLY);
    if (filedes == -1) {
		perror("error opening tmp file for reading");
		return -1;
    }

    // check size of opened test file
    if (0 != fstat(filedes, &stat_buf)) {
		perror("calling fstat() failed\n");
		return -1;
    }
    if (stat_buf.st_size != TEST_FILE_LEN) {
		fprintf(stderr, "error - length of file %s is %d, must be %d\n", test_filename_tmp, (int) stat_buf.st_size, TEST_FILE_LEN);
		return -1;
    }
    
    // read contents of test file, verify that it matches data written to it earlier
    count = read(filedes, data_buf, TEST_FILE_LEN);
    if (count != TEST_FILE_LEN) {
		perror("error reading data from tmp file");
		fprintf(stderr, "error - expected to read %d bytes from %s, but read %d\n", TEST_FILE_LEN, test_filename_tmp, count);
		return -1;
    }
    if (0 != memcmp(data_buf, reference_file_contents, TEST_FILE_LEN)) {
		fprintf(stderr, "error - data read from %s doesn't match expectations\n", test_filename_tmp);
		return -1;
    }
    
    // close test file
    if (0 != close(filedes)) {
		perror("error closing tmp file");
		return -1;
    }
    
    // delete test file
    if (0 != unlink(test_filename_tmp)) {
		perror("error unlinking tmp file");
		return -1;
    }
    
    return 0;
}

int c99_scenario()
{
	unsigned char data_buf[TEST_FILE_LEN];
	FILE* fd;
	int count;
	char *test_filename_tmp = "c99_testfile.tmp";
		
    // create test file
    fd = fopen(test_filename_tmp, "w");
    if (fd == 0) {
		perror("error opening tmp file for writing");
		return -1;
    }
    
    // write data to test file
    count = fwrite(reference_file_contents, 8, TEST_FILE_LEN/8, fd);
    if (count != (TEST_FILE_LEN/8)) {
		perror("error writing data to tmp file");
		fprintf(stderr, "error - expected to write %d 8-byte entries to %s, but wrote %d\n", TEST_FILE_LEN/8, test_filename_tmp, count);
		return -1;
    }
    
    // close test file
    if (0 != fclose(fd)) {
		perror("error closing tmp file");
		return -1;
    }
    
    // open test file for reading
    fd = fopen(test_filename_tmp, "r");
    if (fd == 0) {
		perror("error opening tmp file for reading");
		return -1;
    }
    
    // read contents of test file, verify that they match the data written to it earlier
    count = fread(data_buf, 4, TEST_FILE_LEN/4, fd);
    if (count != (TEST_FILE_LEN/4)) {
		perror("error reading data from tmp file");
		fprintf(stderr, "error - expected to read %d 4-byte entries from %s, but read %d\n", TEST_FILE_LEN/4, test_filename_tmp, count);
		return -1;
    }
    if (0 != memcmp(data_buf, reference_file_contents, TEST_FILE_LEN)) {
		fprintf(stderr, "error - data read from %s doesn't match expectations\n", test_filename_tmp);
		return -1;
    }
    
    // close test file
    if (0 != fclose(fd)) {
		perror("error closing tmp file");
		return -1;
    }
    
    // delete test file
    if (0 != unlink(test_filename_tmp)) {
		perror("error unlinking tmp file");
		return -1;
    }
    
    return 0;
}

int main(unsigned long long id __attribute__ ((unused)), unsigned long long argv __attribute__ ((unused)), unsigned long long env __attribute__ ((unused)), void *secure_env __attribute__ ((unused)))
{
	// increase size of the buffer in open area of local store to be used by PPE-assisted functions
    if (0 != change_ppuassist_buf_len(2048)) {
		printf("Error increasing size of the buffer in open area of local store to be used by PPE-assisted functions\n");
		return -1;	
    }
	
    // run IO sample using POSIX functions
    if (0 != posix_scenario()) {
		printf("[ POSIX scenario FAILED ]\n");
		return -1;
    } else
		printf("[ POSIX scenario SUCCEEDED ]\n");

    // run IO sample using C99 functions
    if (0 != c99_scenario()) {
		printf("[ C99 scenario FAILED ]\n");
		return -1;
    } else
		printf("[ C99 scenario SUCCEEDED ]\n");

    return 0;
}
